@@ -1046,7 +1046,7 @@ macro_rules! __js_serializable_serde_boilerplate {
 #[macro_export]
 macro_rules! __js_deserializable_serde_boilerplate {
     (($($impl_arg:tt)*) ($($kind_arg:tt)*) ($($bounds:tt)*)) => {
-        impl< $($impl_arg),* > TryFrom< $crate::Value > for $($kind_arg)* where $($bounds)* {
+        impl< $($impl_arg),* > $crate::unstable::TryFrom< $crate::Value > for $($kind_arg)* where $($bounds)* {
             type Error = $crate::serde::ConversionError;
             #[inline]
             fn try_from( value: $crate::Value ) -> Result< Self, Self::Error > {
@@ -1059,7 +1059,7 @@ macro_rules! __js_deserializable_serde_boilerplate {
 /// A macro which makes it possible to pass an instance of a given type
 /// implementing Serde's `Serialize` into the [js!](macro.js.html) macro.
 ///
-/// For types defined outside of your crate you can also use the [Serde](struct.Serde.html)
+/// For types defined outside of your crate you can also use the [Serde](serde/struct.Serde.html)
 /// newtype to make them serializable indirectly.
 ///
 /// # Examples
@@ -1129,14 +1129,14 @@ macro_rules! js_serializable {
 ///
 /// let value = js! {
 ///     return {
-///         number: 123,
-///         string: "Hello!"
+///         name: "Bob",
+///         age: 33
 ///     };
 /// };
 ///
-/// let structure: StructureSerializable = value.try_into().unwrap();
-/// assert_eq!( structure.number, 123 );
-/// assert_eq!( structure.string, "Hello!" );
+/// let structure: Person = value.try_into().unwrap();
+/// assert_eq!( structure.name, "Bob" );
+/// assert_eq!( structure.age, 33 );
 /// ```
 ///
 /// This macro also accepts generics just as the [js_serializable!](macro.js_serializable.html) does.
