@@ -103,6 +103,9 @@ pub use webcore::number::Number;
 pub use webcore::object::Object;
 pub use webcore::array::Array;
 
+pub use webcore::unsafe_typed_array::UnsafeTypedArray;
+pub use webcore::once::Once;
+
 #[cfg(feature = "serde")]
 /// A module with serde-related APIs.
 pub mod serde {
@@ -129,6 +132,7 @@ pub mod web {
     pub use webapi::cross_origin_setting::CrossOriginSetting;
     pub use webapi::date::Date;
     pub use webapi::event_target::{IEventTarget, EventTarget, EventListenerHandle};
+    pub use webapi::window::RequestAnimationFrameHandle;
     pub use webapi::node::{INode, Node, CloneKind};
     pub use webapi::element::{IElement, Element};
     pub use webapi::text_node::TextNode;
@@ -141,6 +145,7 @@ pub mod web {
     pub use webapi::location::Location;
     pub use webapi::array_buffer::ArrayBuffer;
     pub use webapi::typed_array::TypedArray;
+    pub use webapi::file_reader::{FileReader, FileReaderResult};
 
     /// A module containing error types.
     pub mod error {
@@ -172,17 +177,22 @@ pub mod web {
 
             ChangeEvent,
             KeypressEvent,
+            KeydownEvent,
+            KeyupEvent,
             ClickEvent,
             DoubleClickEvent,
             FocusEvent,
             BlurEvent,
             HashChangeEvent,
-            LoadEvent,
+            ResourceLoadEvent,
+            ResourceAbortEvent,
+            ResourceErrorEvent,
             ProgressEvent,
             LoadStartEvent,
             LoadEndEvent,
-            AbortEvent,
-            ErrorEvent,
+            ProgressLoadEvent,
+            ProgressAbortEvent,
+            ProgressErrorEvent,
             InputEvent
         };
     }
