@@ -152,6 +152,10 @@ pub fn initialize() {
                 var pointer = HEAPU32[ (address + 4) / 4 ];
                 var deallocator_pointer = HEAPU32[ (address + 8) / 4 ];
                 var output = function() {
+                    if( pointer === 0 ) {
+                        throw new ReferenceError( "Already dropped Rust function called!" );
+                    }
+
                     var args = Module.STDWEB.alloc( 16 );
                     Module.STDWEB.serialize_array( args, arguments );
                     Module.STDWEB.dyncall( "vii", adapter_pointer, [pointer, args] );
@@ -163,7 +167,10 @@ pub fn initialize() {
 
                 output.drop = function() {
                     output.drop = Module.STDWEB.noop;
-                    Module.STDWEB.dyncall( "vi", deallocator_pointer, [pointer] );
+                    var function_pointer = pointer;
+                    pointer = 0;
+
+                    Module.STDWEB.dyncall( "vi", deallocator_pointer, [function_pointer] );
                 };
 
                 return output;
@@ -198,6 +205,30 @@ pub fn initialize() {
                 };
 
                 return output;
+            } else if( kind === 14 ) {
+                var pointer = HEAPU32[ address / 4 ];
+                var length = HEAPU32[ (address + 4) / 4 ];
+                var array_kind = HEAPU32[ (address + 8) / 4 ];
+                var pointer_end = pointer + length;
+
+                switch( array_kind ) {
+                    case 0:
+                        return HEAPU8.subarray( pointer, pointer_end );
+                    case 1:
+                        return HEAP8.subarray( pointer, pointer_end );
+                    case 2:
+                        return HEAPU16.subarray( pointer, pointer_end );
+                    case 3:
+                        return HEAP16.subarray( pointer, pointer_end );
+                    case 4:
+                        return HEAPU32.subarray( pointer, pointer_end );
+                    case 5:
+                        return HEAP32.subarray( pointer, pointer_end );
+                    case 6:
+                        return HEAPF32.subarray( pointer, pointer_end );
+                    case 7:
+                        return HEAPF64.subarray( pointer, pointer_end );
+                }
             }
         };
     };
